import numpy as np
import stan
import matplotlib.pyplot as plt
import seaborn as sb
import arviz as az

Data = [
 [179, 121], [185, 89], [181, 106], [210, 100], [202, 104],
 [197, 91], [227, 107], [208, 101], [188, 99], [189, 112],
 [240, 111], [203, 103], [202, 85], [190, 95], [221, 96],
 [150, 83], [205, 103], [177, 90], [221, 92], [172, 86],
 [190, 87], [218, 101], [205, 100], [208, 120], [224, 98],
 [199, 116], [203, 96], [213, 91], [200, 109], [190, 100],
 [211, 115], [204, 85], [185, 92], [178, 86], [231, 122],
 [206, 84], [197, 93], [236, 93], [213, 106], [205, 103],
 [212, 102], [177, 86], [193, 96], [183, 93], [204, 87],
 [203, 108], [203, 103], [180, 100], [196, 101], [167, 75],
 [202, 106], [182, 98], [225, 109], [219, 115], [205, 102],
 [200, 99], [219, 94], [189, 102], [220, 86], [186, 102],
 [203, 101], [210, 124], [254, 103], [208, 95], [192, 106],
 [176, 91], [173, 97], [195, 100], [189, 98], [220, 115],
 [202, 112], [257, 108], [207, 104], [199, 96], [173, 98],
 [230, 101], [230, 111], [212, 105], [196, 82], [183, 82],
 [230, 103], [207, 112], [225, 101], [192, 104], [175, 83],
 [223, 105], [192, 106], [202, 87], [204, 87], [184, 97],
 [193, 94], [204, 109], [196, 92], [216, 109], [219, 113],
 [256, 117], [210, 86], [217, 103], [206, 87], [213, 105] ]
 
mu = np.mean(Data, axis = 0)
print('mu =\n', mu)
cov = np.cov(np.array(Data).T)
print('cov =\n', cov)
sgms = [cov[0][0]**0.5, cov[1][1]**0.5]
print('sgm =\n', sgms) 


def f_init():
        return {'p_mu1': 0.0, 'p_mu2': 0.0,
                'p_sgm1': 1.0, 'p_sgm2': 1.0, 'p_r': 0.0}
        
stan_data = {'N': len(Data), 'Data': Data, 'm1': mu[0], 'm2': mu[1],
             'sd1': sgms[0], 'sd2': sgms[1]}
with open('TwoVarNormal.stan', 'r') as fstan:
        sm = stan.build(fstan.read(), data = stan_data)
fit = sm.sample(num_chains = 4,  init = [f_init()]*4)

i_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(i_data)
print('Summary =\n',smry)

az.plot_trace(i_data, var_names = ['mu', 'sgm1', 'sgm2', 'r'])
plt.show()

d_fm = fit.to_frame()
print('d_fm keys =\n', d_fm.keys())

samples_mu1 = d_fm['mu.1']  
samples_mu2 = d_fm['mu.2']
med_mu1 = np.median(samples_mu1)
med_mu2 = np.median(samples_mu2)
plt.title('Posterior Distributions', fontsize = 20)
sb.kdeplot(samples_mu1, label = f'$\mu_1$ \nmed.={med_mu1:.2f}')
sb.kdeplot(samples_mu2, label = f'$\mu_2$ \nmed.={med_mu2:.2f}')
plt.xlabel('')
plt.legend(fontsize = 14)
plt.show()

samples_sgm1 = d_fm['sgm1']  
samples_sgm2 = d_fm['sgm2']
med_sgm1 = np.median(samples_sgm1)
med_sgm2 = np.median(samples_sgm2)
plt.title('Posterior Distributions', fontsize = 20)
sb.kdeplot(samples_sgm1, label = f'$\sigma_1$ \nmed.={med_sgm1:.2f}')
sb.kdeplot(samples_sgm2, label = f'$\sigma_2$ \nmed.={med_sgm2:.2f}')
plt.xlabel('')
plt.legend(fontsize = 14)
plt.show()

samples_r = d_fm['r']  
med_r = np.median(samples_r)
plt.title('Posterior Distributions', fontsize = 20)
sb.kdeplot(samples_r, label = f'r \nmed.={med_r:.2f}')
plt.xlabel('r', fontsize = 20)
plt.legend(fontsize = 14)
plt.show()
